#include <stdint.h>

sbit Chip_Select         at RC2_bit;
sbit CS_DIR              at TRISC2_bit;


struct MLX
{
      unsigned int angle;
      unsigned int error_code;
      unsigned char buffer[6];
} mlx;


int read_mlx(){
  uint16_t datavalue = 0;
  uint16_t inverted_datavalue = 0;
  uint16_t checkvalue = 0;

  Chip_Select = 0;
  Delay_us(6);
  SPI3_Write(0xAA);
  SPI3_Write(0xFF);
  Delay_us(45);
  mlx.buffer[3]=SPI3_Read(0xFF);
  mlx.buffer[2]=SPI3_Read(0xFF);
  Delay_us(40);
  mlx.buffer[1]=SPI3_Read(0xFF);
  mlx.buffer[0]=SPI3_Read(0xFF);

  Chip_Select = 1;


  datavalue = ((uint16_t) mlx.buffer[3]) << 8;
  datavalue |= ((uint16_t) mlx.buffer[2]);

  checkvalue = datavalue & 1;   // LSB signifies an error, check if there is an error in measurement

  if (checkvalue == 0 )
     return -1;

  inverted_datavalue = ((uint16_t) mlx.buffer[1]) << 8;
  inverted_datavalue |= ((uint16_t) mlx.buffer[0]);

  // two data bytes are followed by two inverted data bytes, if those allign, then measurement is correct
  if (datavalue != ~inverted_datavalue)
     return -1;

  else
  {
      mlx.angle = datavalue >> 2;
      return 0;
  }

}


void main() 
{
    //Local Declarations
    char txt[20]   = { 0 };
    uint8_t check  = 0;
    float angle    = 0.0;

    CS_DIR = 0;
     
    UART2_Init( 9600 );
    Delay_ms(300);

    UART2_Write_Text("Uart initialized\r\n");

    //SPI
    SPI3_Init_Advanced( _SPI_MASTER, _SPI_8_BIT, 512, _SPI_SS_DISABLE,
                    _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW,
                    _SPI_ACTIVE_2_IDLE );
    Delay_ms(200);
    UART2_Write_Text( "SPI Initialized\r\n" );


    while (1)
    {
        check = read_mlx();
        if ( check == 0)                          // if read was successful, print success and angle
        {
            UART2_Write_Text("Success \r\n");
            angle = (float) mlx.angle;
            angle /= 45.5;
            FloatToStr(angle,txt);
            UART2_Write_Text(txt);
            UART2_Write_Text("\r\n");
         }
         else                                     // else print fail
             UART2_Write_Text("Fail \r\n");

      Delay_ms(400);                // Wait 400ms
    }
}